@@ -1297,8 +1297,8 @@ class Handler(object):
         """
         result = []
         # search for a given pattern in a folder path
-        if pattern == '*':
-            search_str = '*'
+        if '*' in pattern:
+            search_str = pattern
         else:
             search_str = '*' + pattern + '*'
 
