@@ -71,6 +71,7 @@ class Raster(object):
                  shape=None,
                  transform=None,
                  crs_string=None,
+                 nodatavalue=None,
                  datasource=None):
 
         self.array = array
@@ -96,7 +97,7 @@ class Raster(object):
             self.dtype = gdal.GDT_Float32
 
         self.metadict = metadict
-        self.nodatavalue = None
+        self.nodatavalue = nodatavalue
         self.tile_grid = list()
         self.ntiles = None
         self.bounds = None
@@ -871,19 +872,19 @@ class Raster(object):
 
             if xmin < 0:
                 xmin = 0
-            if xmax > self.shape[2]:
-                xmax = self.shape[2]
+            if xmax >= self.shape[2]:
+                xmax = self.shape[2] - 1
             if ymin < 0:
                 ymin = 0
-            if ymax > self.shape[1]:
-                ymax = self.shape[1]
+            if ymax >= self.shape[1]:
+                ymax = self.shape[1] - 1
 
             if xmin >= xmax:
                 raise ImageProcessingError("Image x-size should be greater than 0")
             if ymin >= ymax:
                 raise ImageProcessingError("Image y-size should be greater than 0")
         else:
-            xmin, xmax, ymin, ymax = 0, self.shape[2], 0, self.shape[1]
+            xmin, xmax, ymin, ymax = 0, self.shape[2] - 1, 0, self.shape[1] - 1
 
         return xmin, xmax, ymin, ymax
 
@@ -975,7 +976,7 @@ class Raster(object):
         """
 
         if edge_buffer is None:
-            edge_buffer_x, edge_buffer_y =  0, 0
+            edge_buffer_x, edge_buffer_y = 0, 0
         else:
             edge_buffer_x, edge_buffer_y = edge_buffer
 
@@ -994,32 +995,33 @@ class Raster(object):
         if block_coords is None:
             raise ImageProcessingError("Block coords needed to retrieve tile")
         else:
-            upperleft_x, upperleft_y, tile_rows, tile_cols = block_coords
+            upperleft_x, upperleft_y, tile_cols, tile_rows = block_coords
 
         # raster shape param
         ras_rows, ras_cols = self.shape[1], self.shape[2]
 
         # accounting for number of pixels less than the required size (always >= 0)
         # pixel deficit on left, top, right, and bottom edges respectively
-        pixel_deficit = [(edge_buffer_x - upperleft_x) if (upperleft_x < edge_buffer_x) else 0,
+        pixel_deficit = [(edge_buffer_x - upperleft_x) if (upperleft_x < edge_buffer_x) else 0,  # left
 
-                         (edge_buffer_y - upperleft_y) if (upperleft_y < edge_buffer_y) else 0,
+                         (edge_buffer_y - upperleft_y) if (upperleft_y < edge_buffer_y) else 0,  # top
 
                          (ras_cols - upperleft_x - tile_cols + 1) if
-                         (ras_cols - upperleft_x - tile_cols + 1) < edge_buffer_x else 0,
+                         (ras_cols - upperleft_x - tile_cols + 1) < edge_buffer_x else 0,  # right
 
                          (ras_rows - upperleft_y - tile_rows + 1) if
-                         (ras_rows - upperleft_y - tile_rows + 1) < edge_buffer_y else 0]
+                         (ras_rows - upperleft_y - tile_rows + 1) < edge_buffer_y else 0]  # bottom
 
         # corners
         new_upperleft_x = (upperleft_x - edge_buffer_x) + pixel_deficit[0]
         new_upperleft_y = (upperleft_y - edge_buffer_y) + pixel_deficit[1]
 
-        # new block coordinates
+        # new block coordinates: xoffset, yoffset, xsize, ysize
+        # per https://gdal.org/python/osgeo.gdal.Dataset-class.html#ReadAsArray
         new_block_coords = [new_upperleft_x,
                             new_upperleft_y,
-                            tile_rows + (2 * edge_buffer_y - pixel_deficit[1] + pixel_deficit[3]),
-                            tile_cols + (2 * edge_buffer_x - pixel_deficit[0] + pixel_deficit[2])]
+                            tile_cols + (2 * edge_buffer_x - pixel_deficit[0] + pixel_deficit[2]),
+                            tile_rows + (2 * edge_buffer_y - pixel_deficit[1] + pixel_deficit[3])]
 
         tile_arr = np.zeros((len(bands),
                              new_block_coords[3],
@@ -1115,16 +1117,16 @@ class Raster(object):
             tile_counter += 1
 
     def make_tiles(self,
-                   tile_size_x=1024,
-                   tile_size_y=1024,
+                   tile_xsize=1024,
+                   tile_ysize=1024,
                    out_path=None,
                    edge_buffer=None,
                    **kwargs):
 
         """
         Make and write smaller tile tif files from the raster tif file
-        :param tile_size_y: Tile size along x
-        :param tile_size_x: tile size along y
+        :param tile_xsize: Tile size along x
+        :param tile_ysize: tile size along y
         :param edge_buffer: Buffer along the edges to include while making tiles
                             Tuple of (edge_buf_x, edge_buf_y) in number of pixels
         :param out_path: Output folder for the tiles
@@ -1133,10 +1135,11 @@ class Raster(object):
         if out_path is None:
             out_path = Handler(self.name).dirname
 
-        self.make_tile_grid(tile_size_x, tile_size_y)
-
         tile_counter = 0
-        for tie_pt, tile_arr in self.get_next_tile(edge_buffer=edge_buffer):
+        for tie_pt, tile_arr in self.get_next_tile(tile_xsize=tile_xsize,
+                                                   tile_ysize=tile_ysize,
+                                                   edge_buffer=edge_buffer):
+
             block_coords = self.tile_grid[tile_counter]['block_coords']
             out_file = out_path + Handler(Handler(self.name).basename)\
                 .add_to_filename('_{}'.format('_'.join([str(elem) for elem in block_coords])))
@@ -1154,7 +1157,8 @@ class Raster(object):
                               dtype=self.dtype,
                               shape=tile_arr.shape,
                               transform=out_transform,
-                              crs_string=self.crs_string)
+                              crs_string=self.crs_string,
+                              nodatavalue=self.nodatavalue)
 
             tile_ras.write_to_file(**kwargs)
             tile_ras = None
@@ -1851,6 +1855,8 @@ class MultiRaster:
             if 'outputBounds' in vrt_dict:
                 _ = vrt_dict.pop('xRes')
                 _ = vrt_dict.pop('yRes')
+                ulx, lry, lrx, uly = vrt_dict['outputBounds']
+                vrt_dict['outputBounds'] = [ulx, uly, lrx, lry]
 
             vrt_dict['noData'] = nodata
 
